@@ -3,9 +3,9 @@
 Module to represents whole models
 """
 from typing import Callable
+import logging
 
-import torch.nn as nn
-from torch import Tensor
+from torch import nn, Tensor
 import torch.nn.functional as F
 
 from joeynmt.initialization import initialize_model
@@ -16,6 +16,8 @@ from joeynmt.constants import PAD_TOKEN, EOS_TOKEN, BOS_TOKEN
 from joeynmt.vocabulary import Vocabulary
 from joeynmt.helpers import ConfigurationError
 
+logger = logging.getLogger(__name__)
+
 
 class Model(nn.Module):
     """
@@ -126,7 +128,7 @@ class Model(nn.Module):
                                                       **kwargs)
 
         unroll_steps = trg_input.size(1)
-        assert "decoder_hidden" not in kwargs.keys()
+        assert "decoder_hidden" not in kwargs
         return self._decode(encoder_output=encoder_output,
                             encoder_hidden=encoder_hidden,
                             src_mask=src_mask, trg_input=trg_input,
@@ -180,13 +182,9 @@ class Model(nn.Module):
 
         :return: string representation
         """
-        return "%s(\n" \
-               "\tencoder=%s,\n" \
-               "\tdecoder=%s,\n" \
-               "\tsrc_embed=%s,\n" \
-               "\ttrg_embed=%s)" % (self.__class__.__name__, self.encoder,
-                                    self.decoder, self.src_embed,
-                                    self.trg_embed)
+        return f"{self.__class__.__name__}(\n\tencoder={self.encoder}," \
+                f"\n\tdecoder={self.decoder},\n\tsrc_embed={self.src_embed}," \
+                f"\n\ttrg_embed={self.trg_embed})"
 
 
 class _DataParallel(nn.DataParallel):
@@ -209,6 +207,7 @@ def build_model(cfg: dict = None,
     :param trg_vocab: target vocabulary
     :return: built and initialized model
     """
+    logger.info("Building an encoder-decoder model...")
     src_padding_idx = src_vocab.stoi[PAD_TOKEN]
     trg_padding_idx = trg_vocab.stoi[PAD_TOKEN]
 
@@ -277,4 +276,17 @@ def build_model(cfg: dict = None,
     # custom initialization of model parameters
     initialize_model(model, cfg, src_padding_idx, trg_padding_idx)
 
+    # initialize embeddings from file
+    pretrained_enc_embed_path = cfg["encoder"]["embeddings"].get(
+        "load_pretrained", None)
+    pretrained_dec_embed_path = cfg["decoder"]["embeddings"].get(
+        "load_pretrained", None)
+    if pretrained_enc_embed_path:
+        logger.info("Loading pretraind src embeddings...")
+        model.src_embed.load_from_file(pretrained_enc_embed_path, src_vocab)
+    if pretrained_dec_embed_path and not cfg.get("tied_embeddings", False):
+        logger.info("Loading pretraind trg embeddings...")
+        model.trg_embed.load_from_file(pretrained_dec_embed_path, trg_vocab)
+
+    logger.info("Enc-dec model built.")
     return model
