@@ -9,10 +9,11 @@ import logging
 import numpy as np
 
 import torch
-from torchtext.data import Dataset, Field
+from torchtext.legacy.data import Dataset, Field # pylint: disable=no-name-in-module
 
 from joeynmt.helpers import bpe_postprocess, load_config, make_logger,\
-    get_latest_checkpoint, load_checkpoint, store_attention_plots
+    get_latest_checkpoint, load_checkpoint, store_attention_plots, \
+    expand_reverse_index
 from joeynmt.metrics import bleu, chrf, token_accuracy, sequence_accuracy
 from joeynmt.model import build_model, Model, _DataParallel
 from joeynmt.search import run_batch
@@ -24,7 +25,7 @@ from joeynmt.vocabulary import Vocabulary
 logger = logging.getLogger(__name__)
 
 
-# pylint: disable=too-many-arguments,too-many-locals,no-member
+# pylint: disable=too-many-arguments,too-many-locals,no-member,too-many-branches
 def validate_on_data(model: Model, data: Dataset,
                      batch_size: int,
                      use_cuda: bool, max_output_length: int,
@@ -36,7 +37,8 @@ def validate_on_data(model: Model, data: Dataset,
                      batch_type: str = "sentence",
                      postprocess: bool = True,
                      bpe_type: str = "subword-nmt",
-                     sacrebleu: dict = None) \
+                     sacrebleu: dict = None,
+                     n_best: int = 1) \
         -> (float, float, float, List[str], List[List[str]], List[str],
             List[str], List[List[str]], List[np.array]):
     """
@@ -63,6 +65,7 @@ def validate_on_data(model: Model, data: Dataset,
     :param postprocess: if True, remove BPE segmentation from translations
     :param bpe_type: bpe type, one of {"subword-nmt", "sentencepiece"}
     :param sacrebleu: sacrebleu options
+    :param n_best: Amount of candidates to return
 
     :return:
         - current_valid_score: current validation score [eval_metric],
@@ -102,8 +105,12 @@ def validate_on_data(model: Model, data: Dataset,
             # run as during training to get validation loss (e.g. xent)
 
             batch = batch_class(valid_batch, pad_index, use_cuda=use_cuda)
+            if batch.nseqs < 1:
+                continue
+
             # sort batch now by src length and keep track of order
-            sort_reverse_index = batch.sort_by_src_length()
+            reverse_index = batch.sort_by_src_length()
+            sort_reverse_index = expand_reverse_index(reverse_index, n_best)
 
             # run as during training with teacher forcing
             if compute_loss and batch.trg is not None:
@@ -117,7 +124,8 @@ def validate_on_data(model: Model, data: Dataset,
             # run as during inference to produce translations
             output, attention_scores = run_batch(
                 model=model, batch=batch, beam_size=beam_size,
-                beam_alpha=beam_alpha, max_output_length=max_output_length)
+                beam_alpha=beam_alpha, max_output_length=max_output_length,
+                n_best=n_best)
 
             # sort outputs back to original order
             all_outputs.extend(output[sort_reverse_index])
@@ -125,7 +133,7 @@ def validate_on_data(model: Model, data: Dataset,
                 attention_scores[sort_reverse_index]
                 if attention_scores is not None else [])
 
-        assert len(all_outputs) == len(data)
+        assert len(all_outputs) == len(data) * n_best
 
         if compute_loss and total_ntokens > 0:
             # total validation loss
@@ -201,10 +209,10 @@ def parse_test_args(cfg, mode="test"):
     device = torch.device("cuda" if use_cuda else "cpu")
     if mode == 'test':
         n_gpu = torch.cuda.device_count() if use_cuda else 0
-        k = cfg["testing"].get("beam_size", 1)
-        batch_per_device = batch_size*k // n_gpu if n_gpu > 1 else batch_size*k
-        logger.info("Process device: %s, n_gpu: %d, "
-                    "batch_size per device: %d (with beam_size)",
+        #k = cfg["testing"].get("beam_size", 1)
+        #batch_per_device = batch_size*k // n_gpu if n_gpu > 1 else batch_size*k
+        batch_per_device = batch_size // n_gpu if n_gpu > 1 else batch_size
+        logger.info("Process device: %s, n_gpu: %d, batch_size per device: %d",
                     device, n_gpu, batch_per_device)
         eval_metric = cfg["training"]["eval_metric"]
 
@@ -238,8 +246,8 @@ def parse_test_args(cfg, mode="test"):
         sacrebleu = {"remove_whitespace": True, "tokenize": "13a"}
 
     decoding_description = "Greedy decoding" if beam_size < 2 else \
-        "Beam search decoding with beam size = {} and alpha = {}". \
-            format(beam_size, beam_alpha)
+        f"Beam search decoding with beam size = {beam_size} " \
+        "and alpha = {beam_alpha}"
     tokenizer_info = f"[{sacrebleu['tokenize']}]" \
         if eval_metric == "bleu" else ""
 
@@ -284,6 +292,16 @@ def test(cfg_file,
 
     # load the data
     if datasets is None:
+        # set default vocab path
+        src_vocab_file = os.path.join(model_dir, "src_vocab.txt")
+        trg_vocab_file = os.path.join(model_dir, "trg_vocab.txt")
+        if "src_vocab" not in cfg["data"]:
+            assert os.path.isfile(src_vocab_file), f"{src_vocab_file} not found"
+            cfg["data"]["src_vocab"] = src_vocab_file
+        if "trg_vocab" not in cfg["data"]:
+            assert os.path.isfile(trg_vocab_file), f"{trg_vocab_file} not found"
+            cfg["data"]["trg_vocab"] = trg_vocab_file
+        # load data
         _, dev_data, test_data, src_vocab, trg_vocab = load_data(
             data_cfg=cfg["data"], datasets=["dev", "test"])
         data_to_predict = {"dev": dev_data, "test": test_data}
@@ -299,6 +317,7 @@ def test(cfg_file,
         = parse_test_args(cfg, mode="test")
 
     # load model state from disk
+    logger.info("Loading model from %s", ckpt)
     model_checkpoint = load_checkpoint(ckpt, use_cuda=use_cuda)
 
     # build model and load parameters into it
@@ -340,7 +359,7 @@ def test(cfg_file,
 
         if save_attention:
             if attention_scores:
-                attention_name = "{}.{}.att".format(data_set_name, step)
+                attention_name = f"{data_set_name}.{step}.att"
                 attention_path = os.path.join(model_dir, attention_name)
                 logger.info("Saving attention plots. This might take a while..")
                 store_attention_plots(attentions=attention_scores,
@@ -356,7 +375,7 @@ def test(cfg_file,
                                "Set beam_size to 1 for greedy decoding.")
 
         if output_path is not None:
-            output_path_set = "{}.{}".format(output_path, data_set_name)
+            output_path_set = f"{output_path}.{data_set_name}"
             with open(output_path_set, mode="w", encoding="utf-8") as out_file:
                 for hyp in hypotheses:
                     out_file.write(hyp + "\n")
@@ -366,7 +385,8 @@ def test(cfg_file,
 def translate(cfg_file: str,
               ckpt: str,
               output_path: str = None,
-              batch_class: Batch = Batch) -> None:
+              batch_class: Batch = Batch,
+              n_best: int = 1) -> None:
     """
     Interactive translation function.
     Loads model from checkpoint and translates either the stdin input or
@@ -379,6 +399,7 @@ def translate(cfg_file: str,
     :param ckpt: path to checkpoint to load
     :param output_path: path to output file
     :param batch_class: class type of batch
+    :param n_best: amount of candidates to display
     """
 
     def _load_line_as_data(line):
@@ -387,8 +408,8 @@ def translate(cfg_file: str,
         tmp_name = "tmp"
         tmp_suffix = ".src"
         tmp_filename = tmp_name+tmp_suffix
-        with open(tmp_filename, "w") as tmp_file:
-            tmp_file.write("{}\n".format(line))
+        with open(tmp_filename, "w", encoding="utf-8") as tmp_file:
+            tmp_file.write(f"{line}\n")
 
         test_data = MonoDataset(path=tmp_name, ext=tmp_suffix,
                                 field=src_field)
@@ -409,7 +430,7 @@ def translate(cfg_file: str,
             max_output_length=max_output_length, eval_metric="",
             use_cuda=use_cuda, compute_loss=False, beam_size=beam_size,
             beam_alpha=beam_alpha, postprocess=postprocess,
-            bpe_type=bpe_type, sacrebleu=sacrebleu, n_gpu=n_gpu)
+            bpe_type=bpe_type, sacrebleu=sacrebleu, n_gpu=n_gpu, n_best=n_best)
         return hypotheses
 
     cfg = load_config(cfg_file)
@@ -434,11 +455,9 @@ def translate(cfg_file: str,
 
     tok_fun = lambda s: list(s) if level == "char" else s.split()
 
-    src_field = Field(init_token=None, eos_token=EOS_TOKEN,
-                      pad_token=PAD_TOKEN, tokenize=tok_fun,
-                      batch_first=True, lower=lowercase,
-                      unk_token=UNK_TOKEN,
-                      include_lengths=True)
+    src_field = Field(init_token=None, eos_token=EOS_TOKEN, pad_token=PAD_TOKEN,
+                      tokenize=tok_fun, batch_first=True, lower=lowercase,
+                      unk_token=UNK_TOKEN, include_lengths=True)
     src_field.vocab = src_vocab
 
     # parse test args
@@ -447,6 +466,7 @@ def translate(cfg_file: str,
         bpe_type, sacrebleu, _, _ = parse_test_args(cfg, mode="translate")
 
     # load model state from disk
+    logger.info("Loading model from %s", ckpt)
     model_checkpoint = load_checkpoint(ckpt, use_cuda=use_cuda)
 
     # build model and load parameters into it
@@ -459,18 +479,32 @@ def translate(cfg_file: str,
     if not sys.stdin.isatty():
         # input file given
         test_data = MonoDataset(path=sys.stdin, ext="", field=src_field)
-        hypotheses = _translate_data(test_data)
+        all_hypotheses = _translate_data(test_data)
 
         if output_path is not None:
             # write to outputfile if given
-            output_path_set = "{}".format(output_path)
-            with open(output_path_set, mode="w", encoding="utf-8") as out_file:
-                for hyp in hypotheses:
-                    out_file.write(hyp + "\n")
-            logger.info("Translations saved to: %s.", output_path_set)
+
+            def write_to_file(output_path_set, hypotheses):
+                with open(output_path_set, mode="w", encoding="utf-8") \
+                        as out_file:
+                    for hyp in hypotheses:
+                        out_file.write(hyp + "\n")
+                logger.info("Translations saved to: %s.", output_path_set)
+
+            if n_best > 1:
+                for n in range(n_best):
+                    file_name, file_extension = os.path.splitext(output_path)
+                    write_to_file(
+                        f"{file_name}-{n}" \
+                        f"{file_extension if file_extension else ''}",
+                        [all_hypotheses[i]
+                         for i in range(n, len(all_hypotheses), n_best)]
+                    )
+            else:
+                write_to_file(f"{output_path}", all_hypotheses)
         else:
             # print to stdout
-            for hyp in hypotheses:
+            for hyp in all_hypotheses:
                 print(hyp)
 
     else:
@@ -486,9 +520,11 @@ def translate(cfg_file: str,
 
                 # every line has to be made into dataset
                 test_data = _load_line_as_data(line=src_input)
-
                 hypotheses = _translate_data(test_data)
-                print("JoeyNMT: {}".format(hypotheses[0]))
+
+                print("JoeyNMT: Hypotheses ranked by score")
+                for i, hyp in enumerate(hypotheses):
+                    print(f"JoeyNMT #{i+1}: {hyp}")
 
             except (KeyboardInterrupt, EOFError):
                 print("\nBye.")
